/* TroiaPreview
 * version 1.0 date 2005-03-24
 * Copyright (C) 2003-2005 Stefan Schuermans <1stein@schuermans.info>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 */

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "Images.h"
#include "Textures.h"

//load texture TexNo from image-data
void LoadTexture( unsigned char * pData, long SizeX, long SizeY, long TexNo )
{
  //test pointer
  if( pData == NULL )
    return;

  //load texture into OpenGl
  glBindTexture( GL_TEXTURE_2D, TexNo );
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
  glTexImage2D( GL_TEXTURE_2D, 0, 3, SizeX, SizeY, 0, GL_BGR_EXT, GL_UNSIGNED_BYTE, pData );
}

//unload texture TexNo
void UnloadTexture( long TexNo )
{
  //local variables
  unsigned char BlackPixel[3] = {0, 0, 0};

  //unload texture from OpenGl by loading a one-pixel black one
  glBindTexture( GL_TEXTURE_2D, TexNo );
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
  glTexImage2D( GL_TEXTURE_2D, 0, 3, 1, 1, 0, GL_BGR_EXT, GL_UNSIGNED_BYTE, &BlackPixel );
}

//load texture from bmp-file
int LoadTexBmp( char * FileName, long TexNo, long TexSizeX, long TexSizeY )
{
  //local variables
  unsigned char * pData;

  //load image
  if( !LoadImageResize( FileName, TexSizeX, TexSizeY, &pData, LoadBitmapFile ) )
    //error
    return 0;

  //load data into texture
  LoadTexture( pData, TexSizeX, TexSizeY, TexNo );

  //free data
  free( pData );

  //success
  return 1;
}

//load texture from jpg-file
int LoadTexJpg( char * FileName, long TexNo, long TexSizeX, long TexSizeY )
{
  //local variables
  unsigned char * pData;

  //load bitmap
  if( !LoadImageResize( FileName, TexSizeX, TexSizeY, &pData, LoadJpegFile ) )
    //error
    return 0;

  //load data into texture
  LoadTexture( pData, TexSizeX, TexSizeY, TexNo );

  //free data
  free( pData );

  //success
  return 1;
}

//load texture from bmp-file in pieces
int LoadTexBmpPieces( char * FileName, long TexFirstNo, long TexCntX, long TexCntY, long TexSizeX, long TexSizeY )
{
  //local variables
  unsigned char * * pData;
  long X, Y, I;

  //allocate buffer for texture data array
  pData = (unsigned char * *)malloc( TexCntX * TexCntY * sizeof( unsigned char * ) );
  if( pData == NULL )
    return 0;

  //load bitmap in pieces
  if( !LoadImagePieces( FileName, TexSizeX, TexSizeY, TexCntX, TexCntY, pData, LoadBitmapFile ) )
  {
    free( pData );
    return 0;
  }

  //load bitmap-pieces into textures
  for( Y = 0; Y < TexCntY; Y++ )
  for( X = 0; X < TexCntX; X++ )
  {
    //calculate texture-number
    I = Y * TexCntX + X;
    //load texture
    LoadTexture( pData[I], TexSizeX, TexSizeY, TexFirstNo + I );
    //free data
    free( pData[I] );
  }
  //free texture data array
  free( pData );

  //success
  return 1;
}

//load texture from jpg-file in pieces
int LoadTexJpgPieces( char * FileName, long TexFirstNo, long TexCntX, long TexCntY, long TexSizeX, long TexSizeY )
{
  //local variables
  unsigned char * * pData;
  long X, Y, I;

  //allocate buffer for texture data array
  pData = (unsigned char * *)malloc( TexCntX * TexCntY * sizeof( unsigned char * ) );
  if( pData == NULL )
    return 0;

  //load jpeg in pieces
  if( !LoadImagePieces( FileName, TexSizeX, TexSizeY, TexCntX, TexCntY, pData, LoadJpegFile ) )
  {
    free( pData );
    return 0;
  }

  //load bitmap-pieces into textures
  for( Y = 0; Y < TexCntY; Y++ )
  for( X = 0; X < TexCntX; X++ )
  {
    //calculate texture-number
    I = Y * TexCntX + X;
    //load texture
    LoadTexture( pData[I], TexSizeX, TexSizeY, TexFirstNo + I );
    //free data
    free( pData[I] );
  }
  //free texture data array
  free( pData );

  //success
  return 1;
}
